// app.js
const express = require('express');
const session = require('express-session');
const mysql = require('mysql2');
const bcrypt = require('bcrypt');
const path = require('path');

const app = express();
const PORT = 80;

// Database connection
const db = mysql.createConnection({
    host: 'localhost',
    user: 'testuser',     
    password: 'testpassword',     
    database: 'travelblog'
});

// Middleware for static files - MIME
app.use('/public', express.static(path.join(__dirname, 'public'), {
    setHeaders: function (res, path) {
        if (path.endsWith('.css')) {
            res.set('Content-Type', 'text/css');
        }
    }
}));

// Middleware
app.use(express.urlencoded({ extended: true }));
app.use(express.json());
app.use(express.static(path.join(__dirname, 'public')));
app.set('view engine', 'ejs');
app.set('views', path.join(__dirname, 'views'));

// Session configuration
app.use(session({
    secret: 'supersecretkey',  
    resave: false,
    saveUninitialized: true
}));

/*  

    ROUTES

*/

// Main page
app.get('/', (req, res) => {
    const user_id = req.session.userId;

    const fetchArticlesQuery = `
        SELECT 
            a.article_id, 
            a.title, 
            LEFT(a.content, 150) AS excerpt, 
            d.country,
            COUNT(CASE WHEN v.vote = 'up' THEN 1 END) AS upvotes,
            COUNT(CASE WHEN v.vote = 'down' THEN 1 END) AS downvotes,
            (SELECT vote FROM votes WHERE article_id = a.article_id AND user_id = ?) AS user_vote,
            DATE_FORMAT(a.created_at, '%d.%m.%Y') AS formatted_created_at, 
            DATE_FORMAT(a.updated_at, '%d.%m.%Y') AS formatted_updated_at
        FROM articles a
        LEFT JOIN destinations d ON a.destination_id = d.destination_id
        LEFT JOIN votes v ON a.article_id = v.article_id
        GROUP BY a.article_id
        ORDER BY a.created_at DESC
    `;

    db.query(fetchArticlesQuery, [user_id], (err, articles) => {
        if (err) {
            console.error(err);
            return res.status(500).send('Error fetching articles');
        }

        let username = '';
        if (user_id) {
            db.query('SELECT username FROM users WHERE user_id = ?', [user_id], (err, userResult) => {
                if (err) {
                    console.error(err);
                    return res.status(500).send('Error fetching user');
                }

                if (userResult.length > 0) {
                    username = userResult[0].username;
                }

                const isLoggedIn = !!user_id;
                const isAdminOrDelegate = req.session.role === 'admin' || req.session.role === 'delegate';

                res.render('index', {
                    articles: articles,
                    isLoggedIn: isLoggedIn,
                    isAdminOrDelegate: isAdminOrDelegate,
                    username: username
                });
            });
        } else {
            const isLoggedIn = false;
            const isAdminOrDelegate = false;
            res.render('index', {
                articles: articles,
                isLoggedIn: isLoggedIn,
                isAdminOrDelegate: isAdminOrDelegate,
                username: username
            });
        }
    });
});



// Login route
app.get('/login', (req, res) => {
    res.render('login.ejs');
});
app.post('/login', (req, res) => {
    const { username, password } = req.body;
    const query = 'SELECT * FROM users WHERE username = ?';

    db.query(query, [username], async (err, results) => {
        if (err) return res.sendStatus(500);
        if (results.length === 0) return res.send('User not found');

        const user = results[0];
        const isValid = await bcrypt.compare(password, user.password_hash);

        if (isValid) {
            req.session.userId = user.user_id;
            req.session.username = user.username;
            req.session.role = user.role;
            res.redirect('/');
        } else {
            res.send('Invalid credentials');
        }
    });
});


/*
    DASHBOARD

app.get('/dashboard', (req, res) => {
    if (!req.session.userId) return res.redirect('/'); 
    const query = `
        SELECT Articles.article_id, Articles.title, Articles.content, Destinations.country AS destination
        FROM Articles
        JOIN Destinations ON Articles.destination_id = Destinations.destination_id;
    `;
    
    db.query(query, (err, articles) => {
        if (err) return res.sendStatus(500);
        res.render('dashboard', { articles });
    });
})

*/

// Display article
app.get('/edit-article/:id', (req, res) => {
    const articleId = req.params.id;

    
    const query = `
        SELECT a.*, d.country
        FROM articles a
        LEFT JOIN destinations d ON a.destination_id = d.destination_id
        WHERE a.article_id = ?
    `;

    db.query(query, [articleId], (err, results) => {
        if (err) return res.sendStatus(500);
        if (results.length === 0) return res.send('Article not found');

        const article = results[0];

        
        db.query('SELECT * FROM destinations', (err, destinations) => {
            if (err) return res.sendStatus(500);

           
            res.render('edit-article', { article, destinations });
        });
    });
});


// Update article
app.post('/update-article/:id', (req, res) => {
    const articleId = req.params.id;
    const { title, content, country } = req.body; 

    /* Debugging logs
    console.log("Article ID:", articleId);
    console.log("Title:", title);
    console.log("Content:", content);
    console.log("Country:", country);
    */

    // Validate inputs
    if (!title || !content || !country) {
        return res.status(400).send('Missing required fields');
    }

    const getDestinationQuery = `SELECT destination_id FROM destinations WHERE country = ?`;

    db.execute(getDestinationQuery, [country], (err, results) => {
        if (err) {
            console.error("Error checking destinations:", err);
            return res.status(500).send('Error checking destinations');
        }

        let destinationId;

        if (results.length > 0) {
          
            destinationId = results[0].destination_id;
            console.log("Found existing destination_id:", destinationId); 
            updateArticleWithDestination(articleId, title, content, destinationId);
        } else {
            
            const getMaxIdQuery = `SELECT MAX(destination_id) AS maxId FROM destinations`;

            db.execute(getMaxIdQuery, (errMax, maxResults) => {
                if (errMax) {
                    console.error("Error calculating max destination ID:", errMax);
                    return res.status(500).send('Error calculating destination ID');
                }

                const newId = (maxResults[0].maxId || 9) + 1; 
                 
                const insertDestinationQuery = `INSERT INTO destinations (destination_id, country) VALUES (?, ?)`;

                db.execute(insertDestinationQuery, [newId, country], (errInsert) => {
                    if (errInsert) {
                        console.error("Error inserting new destination:", errInsert);
                        return res.status(500).send('Error saving new destination');
                    }

                    destinationId = newId; 
                    
                    updateArticleWithDestination(articleId, title, content, destinationId);
                });
            });
        }
    });

    function updateArticleWithDestination(articleId, title, content, destinationId) {
        
        if (!destinationId) {
            console.error("Error: destination_id is undefined or null.");
            return res.status(500).send('Invalid destination ID');
        }



        const updateQuery = `
            UPDATE articles
            SET title = ?, content = ?, destination_id = ?
            WHERE article_id = ?
        `;

        db.execute(updateQuery, [title, content, destinationId, articleId], (errUpdate) => {
            if (errUpdate) {
                console.error("Error updating article:", errUpdate);
                return res.status(500).send('Error updating article');
            }

            res.redirect('/');
        });
    }
});


// Delete article
    // Confirm page
app.get('/delete-article/:id', (req, res) => {
    const articleId = req.params.id;

    const query = 'SELECT * FROM articles WHERE article_id = ?';
    db.query(query, [articleId], (err, results) => {
        if (err) {
            console.log(err);
            return res.status(500).send('Error fetching article');
        }
        if (results.length === 0) {
            return res.status(404).send('Article not found');
        }

        const article = results[0];
        res.render('delete-article', { article });
    });
});

    // Delete
app.post('/delete-article/:id', (req, res) => {
    const articleId = req.params.id;

  
    const deleteVotesQuery = 'DELETE FROM votes WHERE article_id = ?';
    db.query(deleteVotesQuery, [articleId], (err, results) => {
        if (err) {
            console.log(err);
            return res.status(500).send('Error deleting related votes');
        }

      
        const deleteArticleQuery = 'DELETE FROM articles WHERE article_id = ?';
        db.query(deleteArticleQuery, [articleId], (err, results) => {
            if (err) {
                console.log(err);
                return res.status(500).send('Error deleting article');
            }

            res.redirect('/');
        });
    });
});

// New Article
app.get('/new-article', (req, res) => {
    const fetchDestinationsQuery = 'SELECT DISTINCT destination_id FROM destinations';
    const fetchTagsQuery = 'SELECT tag_id FROM tags';

    
    db.execute(fetchDestinationsQuery, (errDest, destinations) => {
        if (errDest) {
            console.error(errDest);
            return res.status(500).send('Error fetching destinations');
        }

        db.execute(fetchTagsQuery, (errTags, tags) => {
            if (errTags) {
                console.error(errTags);
                return res.status(500).send('Error fetching tags');
            }

            
            res.render('new-article', { destinations: destinations, tags: tags });
        });
    });
});

// Create article
app.post('/create-article', (req, res) => {
    const { title, content, country } = req.body; 
    const author_id = req.session?.userId; 

    /* Debugging logs
    console.log("Received Data:");
    console.log("Title:", title);
    console.log("Content:", content);
    console.log("Country:", country);
    console.log("Author ID:", author_id);
    */

  
    if (!title || !content || !country || !author_id) {
        console.error("Validation Failed: Missing required fields.");
        return res.status(400).send('Missing required fields');
    }

   
    const getDestinationQuery = `SELECT destination_id FROM destinations WHERE country = ?`;

    db.execute(getDestinationQuery, [country], (err, results) => {
        if (err) {
            console.error("Error querying destinations table:", err);
            return res.status(500).send('Error fetching destination');
        }

        let destinationId;

        if (results.length > 0) {
            
            destinationId = results[0].destination_id;
            console.log("Destination ID found:", destinationId); 
            insertArticleWithDestination(destinationId);
        } else {
            
            const getMaxIdQuery = `SELECT MAX(destination_id) AS maxId FROM destinations`;

            db.execute(getMaxIdQuery, (errMax, maxResults) => {
                if (errMax) {
                    console.error("Error calculating max destination ID:", errMax);
                    return res.status(500).send('Error calculating destination ID');
                }

                
                const newId = (maxResults[0].maxId >= 9 ? maxResults[0].maxId : 9) + 1;
                
                const insertDestinationQuery = `INSERT INTO destinations (destination_id, country) VALUES (?, ?)`;

                db.execute(insertDestinationQuery, [newId, country], (errInsert) => {
                    if (errInsert) {
                        console.error("Error inserting new destination:", errInsert);
                        return res.status(500).send('Error saving new destination');
                    }

          
                    destinationId = newId;
                     
                    insertArticleWithDestination(destinationId);
                });
            });
        }
    });

   
    function insertArticleWithDestination(destination_id) {
        

        
        if (!destination_id) {
            console.error("Error: destination_id is undefined or null.");
            return res.status(500).send('Invalid destination ID');
        }

        const insertArticleQuery = `
            INSERT INTO articles (title, content, author_id, destination_id, created_at, updated_at)
            VALUES (?, ?, ?, ?, NOW(), NOW())
        `;

        db.execute(insertArticleQuery, [title, content, author_id, destination_id], (err, result) => {
            if (err) {
                console.error("Error inserting article:", err);
                return res.status(500).send('Error creating article');
            }

            res.redirect(`/`);
        });
    }
});

// Votes
app.post('/vote', (req, res) => {
    const { article_id, vote } = req.body;
    const user_id = req.session.userId;

    if (!user_id) {
        return res.status(403).send('You must be logged in to vote');
    }

   
    const checkVoteQuery = 'SELECT * FROM votes WHERE article_id = ? AND user_id = ?';
    db.query(checkVoteQuery, [article_id, user_id], (err, results) => {
        if (err) return res.status(500).send('Error checking vote');
        
        if (results.length > 0) {
           
            const updateVoteQuery = 'UPDATE votes SET vote = ? WHERE article_id = ? AND user_id = ?';
            db.query(updateVoteQuery, [vote, article_id, user_id], (err) => {
                if (err) return res.status(500).send('Error updating vote');
                res.redirect('/');
            });
        } else {
           
            const insertVoteQuery = 'INSERT INTO votes (article_id, user_id, vote) VALUES (?, ?, ?)';
            db.query(insertVoteQuery, [article_id, user_id, vote], (err) => {
                if (err) return res.status(500).send('Error inserting vote');
                res.redirect('/');
            });
        }
    });
});

// Logout route
app.get('/logout', (req, res) => {
    req.session.destroy(() => {
        res.redirect('/');
    });
});

// Start server
app.listen(PORT, () => {
    console.log(`Server running on http://localhost:${PORT}`);
});
